import pytest
from frozendict.core import frozendict

class F(frozendict):
    def __new__(cls, *args, **kwargs):
            return super().__new__(cls, *args, **kwargs)

frozendict_class = F

from pathlib import Path

curr_path = Path(__file__)
curr_dir = curr_path.parent

common_path = curr_dir / "common.py"

with open(curr_dir / "subclass_only.py") as f:
    subclass_only_code = f.read()

exec(subclass_only_code)

with open(common_path) as f:
    common_code = f.read()

exec(common_code)



import pytest
import pickle
from copy import copy, deepcopy

##############################################################################
# dict fixtures

@pytest.fixture
def fd_dict():
    return {"Sulla": "Marco", "Hicks": "Bill", frozendict_class({1: 2}): "frozen"}

@pytest.fixture
def fd_dict_eq():
    return {"Hicks": "Bill", "Sulla": "Marco", frozendict_class({1: 2}): "frozen"}

def fd_dict_2_raw():
    return {"Sulla": "Marco", "Hicks": "Bill", "frozen": frozendict_class({1: 2})}

@pytest.fixture
def fd_dict_giulia():
    return {'Marco': 'Sulla', 'Giulia': 'Sulla'}

fd_dict_2 = pytest.fixture(fd_dict_2_raw)

@pytest.fixture
def generator_seq2(fd_dict):
    seq2 = list(fd_dict.items())
    seq2.append(("Sulla", "Mario"))
    return (x for x in seq2)

##############################################################################
# frozendict fixtures

@pytest.fixture
def fd(fd_dict):
    return frozendict_class(fd_dict)

@pytest.fixture
def fd_unhashable():
    return frozendict_class({1: []})

@pytest.fixture
def fd_eq(fd_dict_eq):
    return frozendict_class(fd_dict_eq)

def fd2_raw():
    return frozendict_class(fd_dict_2_raw())

fd2 = pytest.fixture(fd2_raw)

@pytest.fixture
def fd_giulia(fd_dict_giulia):
    return frozendict_class(fd_dict_giulia)

@pytest.fixture
def fd_items(fd_dict):
    return tuple(fd_dict.items())

@pytest.fixture
def fd_empty():
    return frozendict_class()

@pytest.fixture
def module_prefix():
    try:
        sc = subclass
    except NameError:
        sc = False
    
    if sc:
        return ""
    
    return "frozendict."

##############################################################################
# main tests

def test_bool_false(fd_empty):
    assert not fd_empty

def test_constructor_kwargs(fd2, fd_dict_2):
    assert frozendict_class(**fd_dict_2) == fd2

def test_constructor_self(fd):
    assert fd == frozendict_class(fd, Sulla="Marco")

def test_constructor_generator(fd, generator_seq2):
    assert fd == frozendict_class(generator_seq2, Sulla="Marco")

def test_normalget(fd):
    assert fd["Sulla"] == "Marco"

def test_keyerror(fd):
    with pytest.raises(KeyError):
        fd["God"]

def test_len(fd, fd_dict):
    assert len(fd) == len(fd_dict)

def test_in_true(fd):
    assert "Sulla" in fd

def test_not_in_false(fd):
    assert not ("Sulla" not in fd)

def test_in_false(fd):
    assert not ("God" in fd)

def test_not_in_true(fd):
    assert "God" not in fd

def test_bool_true(fd):
    assert fd

def test_copycopy(fd):
    fd3 = copy(fd)
    assert fd3 == fd

def test_deepcopy(fd):
    fd2 = deepcopy(fd)
    assert fd2 == fd

def test_deepcopy_unhashable(fd_unhashable):
    fd2 = deepcopy(fd_unhashable)
    assert fd2 == fd_unhashable
    assert fd2 is not fd_unhashable

def test_not_equal(fd, fd_giulia):
    assert fd != fd_giulia

def test_equals_dict(fd, fd_dict):
    assert fd == fd_dict

@pytest.mark.parametrize("protocol", range(pickle.HIGHEST_PROTOCOL + 1))
def test_pickle(fd, protocol):
    dump = pickle.dumps(fd, protocol=protocol)
    assert dump
    fd_unpickled = pickle.loads(dump)
    assert fd_unpickled == fd

def test_constructor_iterator(fd, fd_items):
    assert frozendict_class(fd_items) == fd

def test_todict(fd, fd_dict):
    assert dict(fd) == fd_dict

def test_get(fd):
    assert fd.get("Sulla") == "Marco"

def test_get_fail(fd):
    default = object()
    assert fd.get("God", default) is default

def test_keys(fd, fd_dict):
    assert tuple(fd.keys()) == tuple(fd_dict.keys())

def test_values(fd, fd_dict):
    assert tuple(fd.values()) == tuple(fd_dict.values())

def test_items(fd, fd_dict):
    assert tuple(fd.items()) == tuple(fd_dict.items())

def test_fromkeys(fd, fd_giulia):
    assert frozendict_class.fromkeys(["Marco", "Giulia"], "Sulla") == fd_giulia

def test_repr(fd, fd_dict, module_prefix):
    classname = frozendict_class.__name__
    repr_fd = repr(fd)
    prefix = module_prefix
    expected_repr = f"{prefix}{classname}({repr(fd_dict)})"
    assert repr_fd == expected_repr

def test_str(fd, fd_dict, module_prefix):
    classname = frozendict_class.__name__
    prefix = module_prefix
    assert str(fd) == f"{prefix}{classname}({repr(fd_dict)})"

def test_format(fd, fd_dict, module_prefix):
    classname = frozendict_class.__name__
    prefix = module_prefix
    assert format(fd) == f"{prefix}{classname}({repr(fd_dict)})"

def test_iter(fd):
    items = []

    for x in iter(fd):
        items.append((x, fd[x]))

    assert tuple(items) == tuple(fd.items())

def test_sum(fd, fd_dict, fd_dict_giulia):
    new_fd = fd | fd_dict_giulia
    new_dict = dict(fd_dict)
    new_dict.update(fd_dict_giulia)
    assert new_fd == new_dict

def test_union(fd_dict, fd_giulia):
    new_fd = frozendict_class(fd_dict)
    id_fd = id(new_fd)
    new_fd |= fd_giulia
    assert id_fd != id(new_fd)
    new_dict = dict(fd_dict)
    new_dict.update(fd_giulia)
    assert new_fd == new_dict
    
    
##############################################################################
# frozendict-only tests

def test_hash(fd, fd_eq):
    assert hash(fd)
    assert hash(fd) == hash(fd_eq)

def test_unhashable_value(fd_unhashable):
    with pytest.raises(TypeError):
        hash(fd_unhashable)

    # hash is cached
    with pytest.raises(TypeError):
        hash(fd_unhashable)

##############################################################################
# immutability tests

def test_normalset(fd):
    with pytest.raises(TypeError):
        fd["Sulla"] = "Silla"

def test_del(fd):
    with pytest.raises(TypeError):
        del fd["Sulla"]

def test_clear(fd):
    with pytest.raises(AttributeError):
        fd.clear()

def test_pop(fd):
    with pytest.raises(AttributeError):
        fd.pop("Sulla")

def test_popitem(fd):
    with pytest.raises(AttributeError):
        fd.popitem()

def test_setdefault(fd):
    with pytest.raises(AttributeError):
        fd.setdefault("Sulla")

def test_update(fd):
    with pytest.raises(AttributeError):
        fd.update({"God": "exists"})

def test_delattr(fd):
    with pytest.raises(AttributeError):
        del fd._initialized



#!/usr/bin/env python3

import frozendict

assert frozendict.c_ext

from frozendict import frozendict, coold
from uuid import uuid4
import pickle
import sys
from pathlib import Path

def getUuid():
    return str(uuid4())

class F(frozendict):
    def __new__(cls, *args, **kwargs):
            return super().__new__(cls, *args, **kwargs)

class C(coold):
    def __new__(cls, *args, **kwargs):
            return super().__new__(cls, *args, **kwargs)

argv = sys.argv
iterations = None
args_num = 1
right_len = args_num + 1
len_argv = len(argv)

if len_argv == right_len:
    try:
        iterations = int(argv[1])
    except Exception:
        pass

if len_argv != right_len or iterations == None:
    err = (
        f"{Path(__file__).name} should have {args_num} and only " + 
        f"{args_num} integer parameter"
    )
    
    print(err, file=sys.stderr)
    exit(1)

key_in = getUuid()
dict_1 = {key_in: 0}
dict_tmp = {getUuid(): i for i in range(1, 1000)}
dict_1.update(dict_tmp)
key_notin = getUuid()
dict_2 = {getUuid(): i for i in range(1000)}
dict_3 = {i: i for i in range(1000)}
dict_unashable = dict_1.copy()
dict_unashable.update({getUuid(): []})
dict_1_items = tuple(dict_1.items())
dict_1_keys = tuple(dict_1.keys())
dict_1_keys_set = set(dict_1_keys)

expressions = (
    'frozendict_class(dict_1)',
    'frozendict_class(dict_3)',
    'frozendict_class()',
    'frozendict_class({})',
    'frozendict_class([])',
    'frozendict_class({}, **{})',
    'frozendict_class(**dict_1)',
    'frozendict_class(dict_1, **dict_2)',
    'frozendict_class(fd_1)',
    'fd_1.copy()',
    'fd_1 == dict_1',
    'fd_1 == fd_1',
    'pickle.loads(pickle.dumps(fd_1))',
    'frozendict_class(dict_1_items)',
    'fd_1.keys()',
    'fd_1.values()',
    'fd_1.items()',
    'frozendict_class.fromkeys(dict_1)',
    'frozendict_class.fromkeys(dict_1, 1)',
    'frozendict_class.fromkeys(dict_1_keys)',
    'frozendict_class.fromkeys(dict_1_keys, 1)',
    'frozendict_class.fromkeys(dict_1_keys_set)',
    'frozendict_class.fromkeys(dict_1_keys_set, 1)',
    'repr(fd_1)',
    'fd_1 | dict_2',
    'hash(fd_1)',
)

codes = (
"""
for x in fd_1:
    pass
""",
"""
for x in fd_1.keys():
    pass
""",
"""
for x in fd_1.values():
    pass
""",
"""
for x in fd_1.items():
    pass
""",
"""
try:
    hash(fd_unashable)
except TypeError:
    pass
""",
)

for frozendict_class in (frozendict, F):
    fd_1 = frozendict_class(dict_1)
    fd_unashable = frozendict_class(dict_unashable)
    
    for expression in expressions:
        print(expression)
        
        for j in range(iterations):
            eval(expression)
    
    for code in codes:
        print(code)
        
        for j in range(iterations):
            exec(code)

coold_expressions = (
    "c1.set(key_in, 1000)", 
    "c1.set(key_notin, 1000)", 
    "c1.delete(key_in)", 
)

for cooold_class in (coold, C):
    c1 = cooold_class(dict_1)
    
    for expression in coold_expressions:
        print(expression)
        
        for j in range(iterations):
            eval(expression)



#!/usr/bin/env python3

try:
    frozendict
    print("builtin")
except NameError:
    from frozendict import coold

class F(coold):
    def __new__(cls, *args, **kwargs):
            return super().__new__(cls, *args, **kwargs)

frozendict = F

def main(number):
    import timeit
    import uuid
    from time import time
    from math import sqrt
    
    def mindev(data, xbar = None):
        if not data:
            raise ValueError("No data")
        
        if xbar == None:
            xbar = min(data)
        
        sigma2 = 0
        
        for x in data:
            sigma2 += (x - xbar) ** 2
        
        N = len(data) - 1
        
        if N < 1:
            N = 1
        
        return sqrt(sigma2 / N)
    
    def autorange(stmt, setup="pass", globals=None, ratio=1000, bench_time=10, number=None):
        if setup == None:
            setup = "pass"
        
        t = timeit.Timer(stmt=stmt, setup=setup, globals=globals)
        break_immediately = False
        
        if number == None:
            a = t.autorange()
            
            num = a[0]
            number = int(num / ratio)
            
            if number < 1:
                number = 1
            
            repeat = int(num / number)
            
            if repeat < 1:
                repeat = 1
        
        else:
            repeat = 1
            break_immediately = True
        
        results = []
        
        data_tmp = t.repeat(number=number, repeat=repeat)
        min_value = min(data_tmp)
        data_min = [min_value]
        
        bench_start = time()
        
        while 1:
            data_min.extend(t.repeat(number=number, repeat=repeat))
            
            if break_immediately or time() - bench_start > bench_time:
                break
        
        data_min.sort()
        xbar = data_min[0]
        i = 0
        
        while i < len(data_min):
            i = len(data_min)
            sigma = mindev(data_min, xbar=xbar)
            
            for i in range(2, len(data_min)):
                if data_min[i] - xbar > 3 * sigma:
                    break
            
            k = i
            
            if i < 5:
                k = 5
            
            del data_min[k:]
        
        return (min(data_min) / number, mindev(data_min, xbar=xbar) / number)
    
    
    def getUuid():
        return str(uuid.uuid4())
    
    
    dictionary_sizes = (5, 1000)
    
    print_tpl = "Name: {name: <25} Size: {size: >4}; Keys: {keys: >3}; Type: {type: >10}; Time: {time:.2e}; Sigma: {sigma:.0e}"
    str_key = '12323f29-c31f-478c-9b15-e7acc5354df9'
    int_key = dictionary_sizes[0] - 2
    
    if int_key < 0:
        int_key = 0

    benchmarks = (
        {"name": "constructor(d)", "code": "klass(d)", "setup": "klass = type(o)", },
        {"name": "constructor(kwargs)", "code": "klass(**d)", "setup": "klass = type(o)", },
        {"name": "constructor(seq2)", "code": "klass(v)", "setup": "klass = type(o); v = tuple(d.items())", },  
        {"name": "constructor(o)", "code": "klass(o)", "setup": "klass = type(o)", },
        {"name": "o.copy()", "code": "o.copy()", "setup": "pass", },
        {"name": "o == o", "code": "o == o", "setup": "pass", }, 
        {"name": "for x in o", "code": "for _ in o: pass", "setup": "pass", },
        {"name": "for x in o.values()", "code": "for _ in values: pass", "setup": "values = o.values()", },  
        {"name": "for x in o.items()", "code": "for _ in items: pass", "setup": "items = o.items()", }, 
        {"name": "pickle.dumps(o)", "code": "dumps(o, protocol=-1)", "setup": "from pickle import dumps", },  
        {"name": "pickle.loads(dump)", "code": "loads(dump)", "setup": "from pickle import loads, dumps; dump = dumps(o, protocol=-1)", },  
        {"name": "class.fromkeys()", "code": "fromkeys(keys)", "setup": "fromkeys = type(o).fromkeys; keys = o.keys()", },  
        {"name": "for x in o.keys()", "code": "for _ in keys: pass", "setup": "keys = o.keys()", },   
        {"name": "for x in iter(o)", "code": "for _ in iter(o): pass", "setup": "pass", }, 
        {"name": "o == d", "code": "o == d", "setup": "pass", },
        {"name": "o.get(key)", "code": "get(key)", "setup": "key = getUuid(); get = o.get", }, 
        {"name": "o[key]", "code": "o[one_key]","setup": "pass", }, 
        {"name": "key in o", "code": "key in o", "setup": "key = getUuid()", },
        {"name": "key not in o", "code": "key not in o", "setup": "key = getUuid()", },
        {"name": "hash(o)", "code": "hash(o)", "setup": "pass", },   
        {"name": "len(o)", "code": "len(o)", "setup": "pass", },  
        {"name": "o.keys()", "code": "keys()", "setup": "keys = o.keys", },  
        {"name": "o.values()", "code": "values()", "setup": "values = o.values", },  
        {"name": "o.items()", "code": "items()", "setup": "items = o.items", },   
        {"name": "iter(o)", "code": "iter(o)", "setup": "pass", }, 
        {"name": "repr(o)", "code": "repr(o)", "setup": "pass", },
        {"name": "str(o)", "code": "str(o)", "setup": "pass", },
        {"name": "set", "code": None, "setup": "val = getUuid()", },
    )
    
    dict_collection = []
    
    for n in dictionary_sizes:
        d1 = {}
        d2 = {}

        for i in range(n-1):
            d1[getUuid()] = getUuid()
            d2[i] = i
        
        d1[str_key] = getUuid()
        d2[999] = 999

        fd1 = frozendict(d1)
        fd2 = frozendict(d2)
        
        dict_collection.append({"str": ((d1, fd1), str_key), "int": ((d2, fd2), int_key)})
        
    for benchmark in benchmarks:
        print("################################################################################")
        
        for dict_collection_entry in dict_collection:
            for (dict_keys, (dicts, one_key)) in dict_collection_entry.items():
        
                if benchmark["name"] == "constructor(kwargs)" and dict_keys == "int":
                    continue

                print("////////////////////////////////////////////////////////////////////////////////")
                
                for o in dicts:
                    if benchmark["name"] == "hash(o)" and type(o) == dict:
                        continue
                    
                    if benchmark["name"] == "set":
                        if type(o) == dict:
                            benchmark["code"] = "o[one_key] = val"
                        else:
                            benchmark["code"] = "o.set(one_key, val)"

                    d = dicts[0]
                    
                    bench_res = autorange(
                        stmt = benchmark["code"], 
                        setup = benchmark["setup"], 
                        globals = {"o": o.copy(), "getUuid": getUuid, "d": d.copy(), "one_key": one_key},
                        number = number, 
                    )

                    print(print_tpl.format(
                        name = "`{}`;".format(benchmark["name"]), 
                        keys = dict_keys, 
                        size = len(o), 
                        type = type(o).__name__, 
                        time = bench_res[0],
                        sigma = bench_res[1],  
                    ))
    
    print("################################################################################")

if __name__ == "__main__":
    import sys

    number = None
    argv = sys.argv
    len_argv = len(argv)
    max_positional_args = 1
    max_len_argv = max_positional_args + 1
    
    if len_argv > max_len_argv:
        raise ValueError(
            ("{name} must not accept more than {nargs} positional " + 
            "command-line parameters").format(name=__name__, nargs=max_positional_args)
        )
    
    number_arg_pos = 1
    
    if len_argv == number_arg_pos + 1:
        number = int(argv[number_arg_pos])
    
    main(number)



#!/usr/bin/env python3

import setuptools
from pathlib import Path
import sys
import platform

name = "frozendict"
main_package_name = "frozendict"
test_dir_name = "test"
readme_filename = "README.md"
version_filename = "VERSION"
main_url = "https://github.com/Marco-Sulla/python-frozendict"
bug_url = "https://github.com/Marco-Sulla/python-frozendict/issues"
author = "Marco Sulla"
author_email = "marcosullaroma@gmail.com"
license  = "LGPL v3"
license_files = "LICENSE.txt"
description = "A simple immutable dictionary"
keywords = "immutable hashable picklable frozendict dict dictionary map Mapping MappingProxyType developers stable utility"
# for pathlib
python_requires = ">=3.6"

classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
    "Programming Language :: Python :: 3 :: Only",
    "Programming Language :: Python :: 3.6", 
    "Natural Language :: English", 
    "Operating System :: OS Independent", 
    "Topic :: Software Development :: Libraries",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Utilities", 
]

curr_path = Path(__file__).resolve()
curr_dir = curr_path.parent

readme_path = curr_dir / readme_filename
readme_content_type = "text/markdown"

long_description = ""

with open(readme_path) as f:
    long_description = f.read()

main_package_path = curr_dir / main_package_name

version = ""
version_path = main_package_path / version_filename

with open(version_path) as f:
    version = f.read()

excluded_packages = (test_dir_name, )
packages = setuptools.find_packages(exclude=excluded_packages)
package_data_filenames = (version_filename, )
package_data = {package_name: package_data_filenames for package_name in packages}

# C extension - START

src_dir_name = "src"
src_base_path = main_package_path / src_dir_name
include_dir_name = "Include"

ext1_name = "_" + name
ext1_fullname = main_package_name + "." + ext1_name
ext1_source1_name = name + "object"
ext1_source1_fullname = ext1_source1_name + ".c"

cpython_include_dir_name = "Include"
cpython_objects_dir_name = "Objects"
cpython_include_internal_name = "internal"
cpython_stringlib_name = "stringlib"
cpython_objects_clinic_name = "clinic"

extra_compile_args = ["-DPY_SSIZE_T_CLEAN", "-DPy_BUILD_CORE"]

pyversion = sys.version_info
cpython_version = f"{pyversion[0]}_{pyversion[1]}"

src_path = src_base_path / cpython_version

cpython_path = src_path / "cpython_src"
cpython_include_path = cpython_path / cpython_include_dir_name
cpython_object_path = cpython_path / cpython_objects_dir_name

include_path = src_path / include_dir_name
cpython_include_path_internal_path = cpython_include_path / cpython_include_internal_name
cpython_stringlib_path = cpython_object_path / cpython_stringlib_name
cpython_objects_clinic_path = cpython_object_path / cpython_objects_clinic_name

cpython_include_dirs = [
    str(include_path), 
    str(cpython_include_path), 
    str(cpython_include_path_internal_path), 
    str(cpython_object_path), 
    str(cpython_stringlib_path), 
    str(cpython_objects_clinic_path), 
]

ext1_source1_path = src_path / ext1_source1_fullname
cpython_dict_path = cpython_object_path / "dictobject.c"

cpython_sources = [
    str(ext1_source1_path), 
    str(cpython_dict_path), 
]

undef_macros = []

argv = sys.argv

if argv[1] == "c_debug":
    undef_macros = ["NDEBUG"]

ext_modules = []

ext_modules.append(setuptools.Extension(
    ext1_fullname,
    sources = cpython_sources,
    include_dirs = cpython_include_dirs,
    extra_compile_args = extra_compile_args,
    undef_macros = undef_macros,
))



# C extension - END

common_setup_args = dict(
    name = name,
    author = author,
    author_email = author_email,
    version = version,
    python_requires = python_requires,
    license  = license,
    license_files = (license_files, ),
    url = main_url,
    
    project_urls = {
        "Bug Reports": bug_url,
        "Source": main_url,
    },
    
    packages = packages,
    package_data = package_data,
    
    description = description,
    long_description = long_description,
    long_description_content_type = readme_content_type,
    
    classifiers = classifiers,
    keywords = keywords,
)

custom_arg = None

if len(argv) > 1 and (argv[1] == "py" or argv[1] == "c_debug"):
    custom_arg = argv[1]
    sys.argv = [sys.argv[0]] + sys.argv[2:]

system = platform.system()

if custom_arg == None and system == "Windows":
    custom_arg = "py"

if custom_arg == "py":
    setuptools.setup(**common_setup_args)
else:
    setuptools.setup(ext_modules = ext_modules, **common_setup_args)



import pytest
import frozendict as frozendict_module
from frozendict import coold as frozendict_class


from pathlib import Path

curr_path = Path(__file__)
curr_dir = curr_path.parent

common_path = curr_dir / "common.py"

with open(common_path) as f:
    common_code = f.read()

exec(common_code)

c_only_path = curr_dir / "c_only.py"

with open(c_only_path) as f:
    c_only_code = f.read()

exec(c_only_code)

with open(curr_dir / "frozendict_only.py") as f:
    frozendict_only_code = f.read()

exec(frozendict_only_code)

def test_coold_set_replace(fd_dict, generator_seq2):
    items = tuple(generator_seq2)
    d2 = dict(items)
    assert fd_dict != d2
    fd2 = frozendict_class(items)
    fd3 = fd2.set("Sulla", "Marco")
    assert fd3 == fd_dict

def test_coold_set_add(fd_dict):
    d2 = dict(fd_dict, a="b")
    assert fd_dict != d2
    fd2 = frozendict_class(fd_dict)
    fd3 = fd2.set("a", "b")
    assert fd3 == d2

def test_coold_del(fd_dict):
    d2 = dict(fd_dict)
    d2["a"] = "b"
    fd2 = frozendict_class(d2)
    fd3 = fd2.delete("a")
    assert fd3 == fd_dict
